﻿<?php

///////////////////////////////////////////////////
// scriptcamp.ir                //
///////////////////////////////////////////////////

require_once "config.php";
require_once "errors.php";
require_once "jdate.php";
require_once "login.class.php";
require_once "user.class.php";
require_once "ads.class.php";
require_once "pm.class.php";
require_once "pay.class.php";
require_once "popup.class.php";
require_once "withdraw.class.php";

//require_once "online_users.php";

class Database {

    //connection =====
            public $connection, $user, $admin, $ads, $pm, $popup, $withdraw;

    public function __construct() {
        $this->user = new user($this);
        $this->admin = new admin($this);
        $this->ads = new ads($this);
        $this->pm = new pm($this);
        $this->pay = new pay($this);
        $this->popup = new popup($this);
        $this->withdraw = new withdraw($this);
    }

    private function error($err, $query = '', $fatal = false) {
        global $errors, $_config;
        if ($_config['debug_mode'])
            echo '<div style="background:#FFDDDD;border:2px solid darkred;padding:5px;">' . ($query != '' ? '<b style="color:red">SQL Error:</b> ' : '') . $err . "<br />" . ($query != '' ? "<b>Query: </b>$query<br />" : '') . "<br />" . Errors::stack(1) . "</div>";
        else
            die("<div style=\"background:#FFDDDD;padding:5px;border:1px red solid;\">$err</div>");

        if ($fatal)
            die('');
    }

    public function connect() {
        global $_config, $_error;
        if ($this->isConnected())
            $this->error($_error['already_connected'], '', true);
        $this->connection = mysql_connect($_config['host'], $_config['username'], $_config['password']) or die($_error['connection_fail']);
    }

    public function disconnect() {
        if ($this->isConnected()) {
            mysql_close($this->connection);
            $this->connection = null;
        }
    }

    public function isConnected() {
        return $this->connection != null;
    }

    public function query($string, $silent = false) {
        global $_error, $_config;
        //check connection
        if (!$this->isConnected())
            $this->error($_error['not_connected_yet'], '', true);

        //replace macros
        $string = str_replace('`$$$`', "`{$_config['dbname']}`", $string);

        //query
        $sql = mysql_query($string);

        //collect results
        if ($sql === false && !$silent)
            $this->error(mysql_error($this->connection), $string, true);

        return $sql;
    }

    public function select($table, $additionalsql = '', $silent = false) {
        global $_config;
        $sql = "select * from `{$_config['dbname']}`.`$table` $additionalsql";
        return $this->query($sql, $silent);
    }

    public function addRecord($table, $rec) {
        //note: this function is sql injection safe
        $names = $this->getRecordNames($rec);
        $values = $this->getRecordValues($rec);
        return $this->query("insert into `$$$`.`$table` ($names) values ($values)");
    }

    public function addBatch($table, $rec) {
        for ($i = 0; $i < count($rec); $i++)
            $this->addRecord($table, $rec[$i]);
    }

    public function add($table, $rec) {
        global $_error;
        if (is_array($rec)) {
            if (count($rec) > 0) {
                $k = array_keys($rec);
                if (is_array($rec[$k[0]])) {
                    //batch
                    $this->addBatch($table, $rec);
                } else {
                    //single
                    return $this->addRecord($table, $rec);
                }
            }
        }
        else
            $this->error($_error['invalid_input'], '', true);
    }

    public function update($table, $record, $where = '', $silent = false) {
        global $_error;

        if (!is_array($record))
            $this->error($_error['invalid_input'], '', true);

        $r = '';
        $k = array_keys($record);
        for ($i = 0; $i < count($record); $i++) {
            $p = "`{$k[$i]}`='" . mysql_real_escape_string($record[$k[$i]]) . "'";
            $r .= ( $r == '' ? '' : ',') . $p;
        }

        $r = "update `$$$`.`$table` set $r $where";
        return $this->query($r, $silent);
    }

    public function delete($table, $additional_sql) {
        return $this->query("delete from `$$$`.`$table` $additional_sql");
    }

    public function deleteAll($table) {
        return $this->query("TRUNCATE TABLE `$$$`.`$table`");
    }

    private function getRecordNames($rec) {
        //note: this function is sql injection safe
        $r = '';
        $k = array_keys($rec);
        for ($i = 0; $i < count($k); $i++) {
            if ($r != '')
                $r .= ',';

            $r .= "`" . mysql_real_escape_string($k[$i]) . "`";
        }
        return $r;
    }

    private function getRecordValues($rec) {
        //note: this function is sql injection safe
        $r = '';
        $k = array_values($rec);
        for ($i = 0; $i < count($k); $i++) {
            if ($r != '')
                $r .= ',';
            $r .= "'" . mysql_real_escape_string($k[$i]) . "'";
        }
        return $r;
    }

    public function affected_rows() {
        return mysql_affected_rows($this->connection);
    }

    public function get_as_array($sql, $from = 0, $to = - 1, $blob_also = false) {
        if ($sql != null) {
            //init
            $count = mysql_num_rows($sql);
            if ($to == -1 || $to > $count)
                $to = $count;
            //get fields
            $f = array();
            for ($i = 0, $count = 0; $i < mysql_num_fields($sql); $i++) {
                $isBlob = strpos(mysql_field_type($sql, $i), "blob");
                if ($blob_also || $isBlob === false)
                    $f[$count++] = mysql_field_name($sql, $i);
            }
            //get result array
            $r = array();
            for ($i = $from; $i < $to; $i++) {//rows
                for ($o = 0; $o < count($f); $o++)//fields
                    $r[$i - $from][$f[$o]] = mysql_result($sql, $i, $f[$o]);
            }
            return $r;
        }
        else
            return array();
    }

    public function get_page_as_array($sql, $page = 1, $item_per_page = 10, $blob_also = false) {
        //figures out page range, and returns it as array using get_as_array function
        $range = $this->get_page_range($sql, $page, $item_per_page);
        return $this->get_as_array($sql, $range[0], $range[1], $blob_also);
    }

    public function get_page_range($sql, $page = 1, $item_per_page = 10) {
        ///returns an array with two values: range start, and range end,
        ///starting from 0
        $num = mysql_num_rows($sql);
        $start = ($page - 1) * $item_per_page;
        $end = min(array($start + $item_per_page, $num));
        return array($start, $end);
    }

    public function get_page_count($sql, $item_per_page = 10) {
        //returns how many pages this query should have
        $num = mysql_num_rows($sql) - 1;
        return max(1, floor($num / $item_per_page) + 1);
    }

    public function getSettings($blob_also = false) {
        //this function works with the table "settings" and reads the data from db and stores it in $settings
        global $settings;
        $sql = $this->select("settings");
        $arr = $this->get_as_array($sql, 0, -1, $blob_also);
        if (count($arr) == 0)
            die("Settings database table is corrupted.");
        $settings = $arr[0];
        return $settings;
    }

    public function setSettings() {
        //this function works with the table "settings" and retrieves the data from $settings and stores it in database
        global $settings;
        return $this->update("settings", $settings);
    }

    public function performSearch($table, $data, $silent = false) {
        /* function help documentation:
         * $table = name of the table to perform search on
         * $silent = whether to show errors or not
         * $data = values of search records
         * 
         * more details about $data:
         * $data = numeric array
         * $data[n] = string array (key names: 'key', 'comparison', 'value')
         * $data[n][0] = name of the table field
         * $data[n][1] = how to compare (> , < , LIKE , = , etc.)
         * $data[n][2] = value to be compared with
         */

        $where = '';
        if (!is_array($data) || (count($data) > 0) && (!isset($data[0]) || !is_array($data[0])))
            $this->error("Invalid parameters sent to Database::performSearch.", '', true);
        //get exact records
        for ($i = 0; $i < count($data); $i++) {
            if ($data[$i][2] == '')
                continue;
            $gen = "`{$data[$i][0]}` {$data[$i][1]} '" . mysql_real_escape_string($data[$i][2]) . "'";
            $where .= ($where == '' ? '' : ' and ') . $gen;
        }
        //produce results
        if ($where != '')
            $where = 'where ' . $where;

        return $this->select($table, $where, $silent);
    }

    public function performEasySearch($table, $exact = array(), $like = array(), $silent = false) {
        //note: data arrays should contain two numeric key values:
        //0: name
        //1: valus
        $data = array();
        for ($i = 0; $i < count($exact); $i++)
            $data[] = array($exact[$i][0], '=', $exact[$i][1]);
        for ($i = 0; $i < count($like); $i++)
            $data[] = array($exact[$i][0], 'like', $exact[$i][1]);
        return $this->performSearch($table, $data, $silent);
    }

    public function getStateList() {
        return array('آذربایجان شرقی', 'آذربایجان غربی', 'اردبیل', 'اصفهان', 'ایلام', 'بوشهر', 'تهران', 'چهارمحال و بختیاری', 'خراسان جنوبی', 'خراسان رضوی', 'خراسان شمالی', 'خوزستان', 'زنجان', 'سمنان', 'سیستان و بلوچستان', 'شيراز', 'قزوین', 'قم', 'کردستان', 'کرمان', 'کرمانشاه', 'کهگیلویه و بویراحمد', 'گلستان', 'گیلان', 'لرستان', 'مازندران', 'مرکزی', 'هرمزگان', 'همدان', 'یزد');
    }

    public function getCityList() {
        return array('آذرشهر', 'اسکو', 'اهر', 'بستان‌آباد', 'بناب', 'تبریز', 'جلفا', 'چاراویماق', 'سراب', 'شبستر', 'عجب‌شیر', 'کلیبر', 'مراغه', 'مرند', 'ملکان', 'میانه', 'ورزقان', 'هریس', 'هشترود', 'ارومیه', 'اشنویه', 'بوکان', 'پیرانشهر', 'تکاب', 'چالدران', 'خوی', 'سردشت', 'سلماس', 'شاهین‌دژ', 'ماکو', 'مهاباد', 'میاندوآب', 'نقده', 'اردبیل', 'بیله‌سوار', 'پارس‌آباد', 'خلخال', 'کوثر', 'گِرمی', 'مِشگین‌شهر', 'نَمین', 'نیر', 'آران و بیدگل', 'اردستان', 'اصفهان', 'برخوار و میمه', 'تیران و کرون', 'چادگان', 'خمینی‌شهر', 'خوانسار', 'سمیرم', 'شهرضا', 'سمیرم سفلی', 'فریدن', 'فریدون‌شهر', 'فلاورجان', 'کاشان', 'گلپایگان', 'لنجان', 'مبارکه', 'نائین', 'نجف‌آباد', 'نطنز', 'آبدانان', 'ایلام', 'ایوان', 'دره‌شهر', 'دهلران', 'شیروان و چرداول', 'مهران', 'بوشهر', 'تنگستان', 'جم', 'دشتستان', 'دشتی', 'دیر', 'دیلم', 'کنگان', 'گناوه', 'اسلام‌شهر', 'پاکدشت', 'تهران', 'دماوند', 'رباط‌کریم', 'ری', 'ساوجبلاغ', 'شمیرانات', 'شهریار', 'فیروزکوه', 'کرج', 'نظرآباد', 'ورامین', 'اردل', 'بروجن', 'شهرکرد', 'فارسان', 'کوهرنگ', 'لردگان', 'بیرجند', 'درمیان', 'سرایان', 'سربیشه', 'فردوس', 'قائنات', 'نهبندان', 'بردسکن', 'تایباد', 'تربت جام', 'تربت حیدریه', 'چناران', 'خلیل‌آباد', 'خواف', 'درگز', 'رشتخوار', 'سبزوار', 'سرخس', 'فریمان', 'قوچان', 'کاشمر', 'کلات', 'گناباد', 'مشهد', 'مه ولات', 'نیشابور', 'اسفراین', 'بجنورد', 'جاجرم', 'شیروان', 'فاروج', 'مانه و سملقان', 'آبادان', 'امیدیه', 'اندیمشک', 'اهواز', 'ایذه', 'باغ‌ملک', 'بندر ماهشهر', 'بهبهان', 'خرمشهر', 'دزفول', 'دشت آزادگان', 'رامشیر', 'رامهرمز', 'شادگان', 'شوش', 'شوشتر', 'گتوند', 'لالی', 'مسجد سلیمان', 'هندیجان', 'ابهر', 'ایجرود', 'خدابنده', 'خرمدره', 'زنجان', 'طارم', 'ماه‌نشان', 'دامغان', 'سمنان', 'شاهرود', 'گرمسار', 'مهدی‌شهر', 'ایرانشهر', 'چابهار', 'خاش', 'دلگان', 'زابل', 'زاهدان', 'زهک', 'سراوان', 'سرباز', 'کنارک', 'نیک‌شهر', 'آباده', 'ارسنجان', 'استهبان', 'اقلید', 'بوانات', 'پاسارگاد', 'جهرم', 'خرم‌بید', 'خنج', 'داراب', 'زرین‌دشت', 'سپیدان', 'شیراز', 'فراشبند', 'فسا', 'فیروزآباد', 'قیر و کارزین', 'کازرون', 'لارستان', 'لامِرد', 'مرودشت', 'ممسنی', 'مهر', 'نی‌ریز', 'آبیک', 'البرز', 'بوئین‌زهرا', 'تاکستان', 'قزوین', 'قم', 'بانه', 'بیجار', 'دیواندره', 'سروآباد', 'سقز', 'سنندج', 'قروه', 'کامیاران', 'مریوان', 'بافت', 'بردسیر', 'بم', 'جیرفت', 'راور', 'رفسنجان', 'رودبار جنوب', 'زرند', 'سیرجان', 'شهر بابک', 'عنبرآباد', 'قلعه گنج', 'کرمان', 'کوهبنان', 'کهنوج', 'منوجان', 'اسلام‌آباد غرب', 'پاوه', 'ثلاث باباجانی', 'جوانرود', 'دالاهو', 'روانسر', 'سرپل ذهاب', 'سنقر', 'صحنه', 'قصر شیرین', 'کرمانشاه', 'کنگاور', 'گیلان غرب', 'هرسین', 'بویراحمد', 'بهمئی', 'دنا', 'کهگیلویه', 'گچساران', 'آزادشهر', 'آزادشهر', 'آق‌قلا', 'بندر گز', 'ترکمن', 'رامیان', 'علی‌آباد', 'کردکوی', 'کلاله', 'گرگان', 'گنبد کاووس', 'مراوه‌تپه', 'مینودشت', 'آستارا', 'آستانه اشرفیه', 'اَملَش', 'بندر انزلی', 'رشت', 'رضوانشهر', 'رودبار', 'رودسر', 'سیاهکل', 'شَفت', 'صومعه‌سرا', 'طوالش', 'فومَن', 'لاهیجان', 'لنگرود', 'ماسال', 'ازنا', 'اليگودرز', 'بروجرد', 'پل‌دختر', 'خرم‌آباد', 'دورود', 'دلفان', 'سلسله', 'کوهدشت', 'آمل', 'بابل', 'بابلسر', 'بهشهر', 'تنکابن', 'جويبار', 'چالوس', 'رامسر', 'ساري', 'سوادکوه', 'قائم‌شهر', 'گلوگاه', 'محمودآباد', 'نکا', 'نور', 'نوشهر', 'آشتيان', 'اراک', 'تفرش', 'خمين', 'دليجان', 'زرنديه', 'ساوه', 'شازند', 'کميجان', 'محلات', 'ابوموسي', 'بستک', 'بندر عباس', 'بندر لنگه', 'جاسک', 'حاجي‌آباد', 'خمير', 'رودان', 'قشم', 'گاوبندي', 'ميناب', 'اسدآباد', 'بهار', 'تويسرکان', 'رزن', 'کبودرآهنگ', 'ملاير', 'نهاوند', 'همدان', 'ابرکوه', 'اردکان', 'بافق', 'تفت', 'خاتم', 'صدوق', 'طبس', 'مهريز', 'مِيبُد', 'يزد');
    }

    public function getCityState($cityindex) {
        //city id is started from 0
        $borders = array(19, 33, 42, 63, 70, 79, 92, 98, 105, 124, 130, 150, 157, 162, 173, 197, 202, 203, 212, 228, 242, 247, 260, 276, 285, 301, 311, 322, 330);
        $idx = 0;
        $last = 0;
        foreach ($borders as $b) {
            if ($cityindex >= $last && $cityindex < $b)
                return $idx;
            $idx++;
            $last = $b;
        }
        return $idx;
    }

    public function getStateCities($stateindex) {
        $arr = $this->getCityList();
        $r = array();
        foreach ($arr as $i => $a) {
            if ($this->getCityState($i) == $stateindex)
                $r[$i] = $a;
        }
        return $r;
    }

}

?>